@@ -66,43 +66,43 @@ namespace Ogre {
     protected:
         /// Whether the high-level program (and it's parameter defs) is loaded
         bool mHighLevelLoaded;
+        /// Have we built the name->index parameter map yet?
+        bool mConstantDefsBuilt;
         /// The underlying assembler program
         GpuProgramPtr mAssemblerProgram;
-        /// Have we built the name->index parameter map yet?
-        mutable bool mConstantDefsBuilt;
         /// Preprocessor options
         String mPreprocessorDefines;
 
         /// in-situ parsing of defines
         static std::vector<std::pair<const char*, const char*>> parseDefines(String& defines);
 
+        String appendBuiltinDefines(String defines);
+
         /// Internal load high-level portion if not loaded
         virtual void loadHighLevel(void);
         /// Internal unload high-level portion if loaded
         virtual void unloadHighLevel(void);
-        /** Internal load implementation, loads just the high-level portion, enough to 
-            get parameters.
-        */
-        virtual void loadHighLevelImpl(void);
         /** Internal method for creating an appropriate low-level program from this
         high-level program, must be implemented by subclasses. */
         virtual void createLowLevelImpl(void) = 0;
         /// Internal unload implementation, must be implemented by subclasses
         virtual void unloadHighLevelImpl(void) = 0;
         /// Populate the passed parameters with name->index map
-        virtual void populateParameterNames(GpuProgramParametersSharedPtr params);
+        void populateParameterNames(GpuProgramParametersSharedPtr params);
         /** Build the constant definition map, must be overridden.
         @note The implementation must fill in the (inherited) mConstantDefs field at a minimum, 
             and if the program requires that parameters are bound using logical 
-            parameter indexes then the mFloatLogicalToPhysical and mIntLogicalToPhysical
+            parameter indexes then the mLogicalToPhysical and mIntLogicalToPhysical
             maps must also be populated.
         */
-        virtual void buildConstantDefinitions() const = 0;
+        virtual void buildConstantDefinitions() = 0;
 
         /** @copydoc Resource::loadImpl */
         void loadImpl();
         /** @copydoc Resource::unloadImpl */
         void unloadImpl();
+
+        void setupBaseParamDictionary();
     public:
         /** Constructor, should be used only by factory classes. */
         HighLevelGpuProgram(ResourceManager* creator, const String& name, ResourceHandle handle,
@@ -125,7 +125,7 @@ namespace Ogre {
         @note
         Only available if this parameters object has named parameters.
         */
-        const GpuNamedConstants& getConstantDefinitions() const;
+        const GpuNamedConstants& getConstantDefinitions() override;
 
         virtual size_t calculateSize(void) const;
 
@@ -135,7 +135,7 @@ namespace Ogre {
         const String& getPreprocessorDefines(void) const { return mPreprocessorDefines; }
 
         /// Scan the source for \#include and replace with contents from OGRE resources
-        static String _resolveIncludes(const String& source, Resource* resourceBeingLoaded, const String& fileName);
+        static String _resolveIncludes(const String& source, Resource* resourceBeingLoaded, const String& fileName, bool supportsFilename = false);
     };
     /** @} */
     /** @} */
