@@ -1,50 +1,113 @@
 /*
 -----------------------------------------------------------------------------
-This source file is part of OGRE 
+This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under 
-the terms of the GNU Lesser General Public License as published by the Free Software 
-Foundation; either version 2 of the License, or (at your option) any later 
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT 
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with 
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple 
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to 
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
 
 #include "OgreBillboardParticleRenderer.h"
 #include "OgreParticle.h"
-#include "OgreStringConverter.h"
+#include "OgreBillboard.h"
 
 namespace Ogre {
-    String rendererTypeName = "billboard";
+    static String rendererTypeName = "billboard";
+
+    /** Command object for billboard type (see ParamCommand).*/
+    class _OgrePrivate CmdBillboardType : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for billboard origin (see ParamCommand).*/
+    class _OgrePrivate CmdBillboardOrigin : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for billboard rotation type (see ParamCommand).*/
+    class _OgrePrivate CmdBillboardRotationType : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for common direction (see ParamCommand).*/
+    class _OgrePrivate CmdCommonDirection : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for common up-vector (see ParamCommand).*/
+    class _OgrePrivate CmdCommonUpVector : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for point rendering (see ParamCommand).*/
+    class _OgrePrivate CmdPointRendering : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    /** Command object for accurate facing(see ParamCommand).*/
+    class _OgrePrivate CmdAccurateFacing : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const;
+        void doSet(void* target, const String& val);
+    };
+    static CmdBillboardType msBillboardTypeCmd;
+    static CmdBillboardOrigin msBillboardOriginCmd;
+    static CmdBillboardRotationType msBillboardRotationTypeCmd;
+    static CmdCommonDirection msCommonDirectionCmd;
+    static CmdCommonUpVector msCommonUpVectorCmd;
+    static CmdPointRendering msPointRenderingCmd;
+    static CmdAccurateFacing msAccurateFacingCmd;
+
+    static class CmdStacksAndSlices : public ParamCommand
+    {
+    public:
+        String doGet(const void* target) const
+        {
+            return StringConverter::toString(
+                static_cast<const BillboardParticleRenderer*>(target)->getTextureStacksAndSlices());
+        }
+        void doSet(void* target, const String& val)
+        {
+            Vector2 tmp = StringConverter::parseVector2(val);
+            static_cast<BillboardParticleRenderer*>(target)->setTextureStacksAndSlices(tmp.x, tmp.y);
+        }
+    } msStacksAndSlicesCmd;
 
     //-----------------------------------------------------------------------
-    BillboardParticleRenderer::CmdBillboardType BillboardParticleRenderer::msBillboardTypeCmd;
-    BillboardParticleRenderer::CmdBillboardOrigin BillboardParticleRenderer::msBillboardOriginCmd;
-    BillboardParticleRenderer::CmdBillboardRotationType BillboardParticleRenderer::msBillboardRotationTypeCmd;
-    BillboardParticleRenderer::CmdCommonDirection BillboardParticleRenderer::msCommonDirectionCmd;
-    BillboardParticleRenderer::CmdCommonUpVector BillboardParticleRenderer::msCommonUpVectorCmd;
-    BillboardParticleRenderer::CmdPointRendering BillboardParticleRenderer::msPointRenderingCmd;
-	BillboardParticleRenderer::CmdAccurateFacing BillboardParticleRenderer::msAccurateFacingCmd;
-    //-----------------------------------------------------------------------
-    BillboardParticleRenderer::BillboardParticleRenderer()
+    BillboardParticleRenderer::BillboardParticleRenderer() : mStacksSlices(1, 1)
     {
         if (createParamDictionary("BillboardParticleRenderer"))
         {
@@ -67,41 +130,45 @@ namespace Ogre {
 
             dict->addParameter(ParameterDef("billboard_rotation_type", 
                 "This setting controls the billboard rotation type. "
-				"'vertex' means rotate the billboard's vertices around their facing direction."
+                "'vertex' means rotate the billboard's vertices around their facing direction."
                 "'texcoord' means rotate the billboard's texture coordinates. Default value is 'texcoord'.",
                 PT_STRING),
                 &msBillboardRotationTypeCmd);
 
             dict->addParameter(ParameterDef("common_direction", 
                 "Only useful when billboard_type is oriented_common or perpendicular_common. "
-				"When billboard_type is oriented_common, this parameter sets the common orientation for "
-				"all particles in the set (e.g. raindrops may all be oriented downwards). "
-				"When billboard_type is perpendicular_common, this parameter sets the perpendicular vector for "
-				"all particles in the set (e.g. an aureola around the player and parallel to the ground).",
+                "When billboard_type is oriented_common, this parameter sets the common orientation for "
+                "all particles in the set (e.g. raindrops may all be oriented downwards). "
+                "When billboard_type is perpendicular_common, this parameter sets the perpendicular vector for "
+                "all particles in the set (e.g. an aureola around the player and parallel to the ground).",
                 PT_VECTOR3),
                 &msCommonDirectionCmd);
 
             dict->addParameter(ParameterDef("common_up_vector",
                 "Only useful when billboard_type is perpendicular_self or perpendicular_common. This "
-				"parameter sets the common up-vector for all particles in the set (e.g. an aureola around "
-				"the player and parallel to the ground).",
+                "parameter sets the common up-vector for all particles in the set (e.g. an aureola around "
+                "the player and parallel to the ground).",
                 PT_VECTOR3),
                 &msCommonUpVectorCmd);
             dict->addParameter(ParameterDef("point_rendering",
                 "Set whether or not particles will use point rendering "
-				"rather than manually generated quads. This allows for faster "
-				"rendering of point-oriented particles although introduces some "
-				"limitations too such as requiring a common particle size."
-				"Possible values are 'true' or 'false'.",
+                "rather than manually generated quads. This allows for faster "
+                "rendering of point-oriented particles although introduces some "
+                "limitations too such as requiring a common particle size."
+                "Possible values are 'true' or 'false'.",
                 PT_BOOL),
                 &msPointRenderingCmd);
-			dict->addParameter(ParameterDef("accurate_facing",
-				"Set whether or not particles will be oriented to the camera "
-				"based on the relative position to the camera rather than just "
-				"the camera direction. This is more accurate but less optimal. "
-				"Cannot be combined with point rendering.",
-				PT_BOOL),
-				&msAccurateFacingCmd);
+            dict->addParameter(ParameterDef("accurate_facing",
+                "Set whether or not particles will be oriented to the camera "
+                "based on the relative position to the camera rather than just "
+                "the camera direction. This is more accurate but less optimal. "
+                "Cannot be combined with point rendering.",
+                PT_BOOL),
+                &msAccurateFacingCmd);
+
+            dict->addParameter(ParameterDef("texture_sheet_size", "",
+                PT_UNSIGNED_INT),
+                &msStacksAndSlicesCmd);
         }
 
         // Create billboard set
@@ -112,6 +179,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     BillboardParticleRenderer::~BillboardParticleRenderer()
     {
+        // mBillboardSet is never actually attached to a node, we just passthrough
+        // based on the particle system's attachment. So manually notify that it's
+        // no longer attached.
+        mBillboardSet->_notifyAttached(0);
         OGRE_DELETE  mBillboardSet;
     }
     //-----------------------------------------------------------------------
@@ -121,160 +192,49 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     void BillboardParticleRenderer::_updateRenderQueue(RenderQueue* queue, 
-        std::list<Particle*>& currentParticles, bool cullIndividually)
+        std::vector<Particle*>& currentParticles, bool cullIndividually)
     {
         mBillboardSet->setCullIndividually(cullIndividually);
 
         // Update billboard set geometry
         mBillboardSet->beginBillboards(currentParticles.size());
         Billboard bb;
-        for (std::list<Particle*>::iterator i = currentParticles.begin();
-            i != currentParticles.end(); ++i)
+
+        for (Particle* p : currentParticles)
         {
-            Particle* p = *i;
-            bb.mPosition = p->position;
-			if (mBillboardSet->getBillboardType() == BBT_ORIENTED_SELF ||
-				mBillboardSet->getBillboardType() == BBT_PERPENDICULAR_SELF)
-			{
-				// Normalise direction vector
-				bb.mDirection = p->direction;
-				bb.mDirection.normalise();
-			}
-            bb.mColour = p->colour;
-            bb.mRotation = p->rotation;
-            // Assign and compare at the same time
-            if (bb.mOwnDimensions = p->mOwnDimensions)
+            bb.mPosition = p->mPosition;
+
+            if (mBillboardSet->getBillboardType() == BBT_ORIENTED_SELF ||
+                mBillboardSet->getBillboardType() == BBT_PERPENDICULAR_SELF)
+            {
+                // Normalise direction vector
+                bb.mDirection = p->mDirection;
+                bb.mDirection.normalise();
+            }
+            bb.mColour = p->mColour;
+            bb.mRotation = p->mRotation;
+            bb.mTexcoordIndex = p->mTexcoordIndex;
+            bb.mOwnDimensions = p->mWidth != mBillboardSet->getDefaultWidth() ||
+                                p->mHeight != mBillboardSet->getDefaultHeight();
+            if (bb.mOwnDimensions)
             {
                 bb.mWidth = p->mWidth;
                 bb.mHeight = p->mHeight;
             }
             mBillboardSet->injectBillboard(bb);
-
         }
-        
+
         mBillboardSet->endBillboards();
 
         // Update the queue
         mBillboardSet->_updateRenderQueue(queue);
     }
-	//---------------------------------------------------------------------
-	void BillboardParticleRenderer::visitRenderables(Renderable::Visitor* visitor, 
-		bool debugRenderables)
-	{
-		mBillboardSet->visitRenderables(visitor, debugRenderables);
-	}
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_setMaterial(MaterialPtr& mat)
+
+    void BillboardParticleRenderer::_notifyBoundingBox(const AxisAlignedBox& aabb)
     {
-        mBillboardSet->setMaterialName(mat->getName());
+        mBillboardSet->setBounds(aabb, Math::boundingRadiusFromAABB(aabb));
     }
     //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::setBillboardType(BillboardType bbt)
-    {
-        mBillboardSet->setBillboardType(bbt);
-    }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setUseAccurateFacing(bool acc)
-	{
-		mBillboardSet->setUseAccurateFacing(acc);
-	}
-	//-----------------------------------------------------------------------
-	bool BillboardParticleRenderer::getUseAccurateFacing(void) const
-	{
-		return mBillboardSet->getUseAccurateFacing();
-	}
-    //-----------------------------------------------------------------------
-    BillboardType BillboardParticleRenderer::getBillboardType(void) const
-    {
-        return mBillboardSet->getBillboardType();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::setBillboardRotationType(BillboardRotationType rotationType)
-    {
-        mBillboardSet->setBillboardRotationType(rotationType);
-    }
-    //-----------------------------------------------------------------------
-    BillboardRotationType BillboardParticleRenderer::getBillboardRotationType(void) const
-    {
-        return mBillboardSet->getBillboardRotationType();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::setCommonDirection(const Vector3& vec)
-    {
-        mBillboardSet->setCommonDirection(vec);
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& BillboardParticleRenderer::getCommonDirection(void) const
-    {
-        return mBillboardSet->getCommonDirection();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::setCommonUpVector(const Vector3& vec)
-    {
-        mBillboardSet->setCommonUpVector(vec);
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& BillboardParticleRenderer::getCommonUpVector(void) const
-    {
-        return mBillboardSet->getCommonUpVector();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyCurrentCamera(Camera* cam)
-    {
-        mBillboardSet->_notifyCurrentCamera(cam);
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyParticleRotated(void)
-    {
-        mBillboardSet->_notifyBillboardRotated();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyDefaultDimensions(Real width, Real height)
-    {
-        mBillboardSet->setDefaultDimensions(width, height);
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyParticleResized(void)
-    {
-        mBillboardSet->_notifyBillboardResized();
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyParticleQuota(size_t quota)
-    {
-        mBillboardSet->setPoolSize(quota);
-    }
-    //-----------------------------------------------------------------------
-    void BillboardParticleRenderer::_notifyAttached(Node* parent, bool isTagPoint)
-    {
-        mBillboardSet->_notifyAttached(parent, isTagPoint);
-    }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setRenderQueueGroup(uint8 queueID)
-	{
-		assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
-		mBillboardSet->setRenderQueueGroup(queueID);
-	}
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setKeepParticlesInLocalSpace(bool keepLocal)
-	{
-		mBillboardSet->setBillboardsInWorldSpace(!keepLocal);
-	}
-    //-----------------------------------------------------------------------
-    SortMode BillboardParticleRenderer::_getSortMode(void) const
-    {
-        return mBillboardSet->_getSortMode();
-    }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setPointRenderingEnabled(bool enabled)
-	{
-		mBillboardSet->setPointRenderingEnabled(enabled);
-	}
-	//-----------------------------------------------------------------------
-	bool BillboardParticleRenderer::isPointRenderingEnabled(void) const
-	{
-		return mBillboardSet->isPointRenderingEnabled();
-	}
-    //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     const String& BillboardParticleRendererFactory::getType() const
@@ -295,7 +255,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdBillboardType::doGet(const void* target) const
+    String CmdBillboardType::doGet(const void* target) const
     {
         BillboardType t = static_cast<const BillboardParticleRenderer*>(target)->getBillboardType();
         switch(t)
@@ -317,7 +277,7 @@ namespace Ogre {
         // Compiler nicety
         return "";
     }
-    void BillboardParticleRenderer::CmdBillboardType::doSet(void* target, const String& val)
+    void CmdBillboardType::doSet(void* target, const String& val)
     {
         BillboardType t;
         if (val == "point")
@@ -350,7 +310,7 @@ namespace Ogre {
         static_cast<BillboardParticleRenderer*>(target)->setBillboardType(t);
     }
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdBillboardOrigin::doGet(const void* target) const
+    String CmdBillboardOrigin::doGet(const void* target) const
     {
         BillboardOrigin o = static_cast<const BillboardParticleRenderer*>(target)->getBillboardOrigin();
         switch (o)
@@ -375,9 +335,9 @@ namespace Ogre {
             return "bottom_right";
         }
         // Compiler nicety
-        return StringUtil::BLANK;
+        return BLANKSTRING;
     }
-    void BillboardParticleRenderer::CmdBillboardOrigin::doSet(void* target, const String& val)
+    void CmdBillboardOrigin::doSet(void* target, const String& val)
     {
         BillboardOrigin o;
         if (val == "top_left")
@@ -408,7 +368,7 @@ namespace Ogre {
         static_cast<BillboardParticleRenderer*>(target)->setBillboardOrigin(o);
     }
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdBillboardRotationType::doGet(const void* target) const
+    String CmdBillboardRotationType::doGet(const void* target) const
     {
         BillboardRotationType r = static_cast<const BillboardParticleRenderer*>(target)->getBillboardRotationType();
         switch(r)
@@ -419,9 +379,9 @@ namespace Ogre {
             return "texcoord";
         }
         // Compiler nicety
-        return StringUtil::BLANK;
+        return BLANKSTRING;
     }
-    void BillboardParticleRenderer::CmdBillboardRotationType::doSet(void* target, const String& val)
+    void CmdBillboardRotationType::doSet(void* target, const String& val)
     {
         BillboardRotationType r;
         if (val == "vertex")
@@ -438,49 +398,50 @@ namespace Ogre {
         static_cast<BillboardParticleRenderer*>(target)->setBillboardRotationType(r);
     }
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdCommonDirection::doGet(const void* target) const
+    String CmdCommonDirection::doGet(const void* target) const
     {
         return StringConverter::toString(
             static_cast<const BillboardParticleRenderer*>(target)->getCommonDirection() );
     }
-    void BillboardParticleRenderer::CmdCommonDirection::doSet(void* target, const String& val)
+    void CmdCommonDirection::doSet(void* target, const String& val)
     {
         static_cast<BillboardParticleRenderer*>(target)->setCommonDirection(
             StringConverter::parseVector3(val));
     }
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdCommonUpVector::doGet(const void* target) const
+    String CmdCommonUpVector::doGet(const void* target) const
     {
         return StringConverter::toString(
             static_cast<const BillboardParticleRenderer*>(target)->getCommonUpVector() );
     }
-    void BillboardParticleRenderer::CmdCommonUpVector::doSet(void* target, const String& val)
+    void CmdCommonUpVector::doSet(void* target, const String& val)
     {
         static_cast<BillboardParticleRenderer*>(target)->setCommonUpVector(
             StringConverter::parseVector3(val));
     }
     //-----------------------------------------------------------------------
-    String BillboardParticleRenderer::CmdPointRendering::doGet(const void* target) const
+    String CmdPointRendering::doGet(const void* target) const
     {
         return StringConverter::toString(
             static_cast<const BillboardParticleRenderer*>(target)->isPointRenderingEnabled() );
     }
-    void BillboardParticleRenderer::CmdPointRendering::doSet(void* target, const String& val)
+    void CmdPointRendering::doSet(void* target, const String& val)
     {
         static_cast<BillboardParticleRenderer*>(target)->setPointRenderingEnabled(
             StringConverter::parseBool(val));
     }
-	//-----------------------------------------------------------------------
-	String BillboardParticleRenderer::CmdAccurateFacing::doGet(const void* target) const
-	{
-		return StringConverter::toString(
-			static_cast<const BillboardParticleRenderer*>(target)->getUseAccurateFacing() );
-	}
-	void BillboardParticleRenderer::CmdAccurateFacing::doSet(void* target, const String& val)
-	{
-		static_cast<BillboardParticleRenderer*>(target)->setUseAccurateFacing(
-			StringConverter::parseBool(val));
-	}
+    //-----------------------------------------------------------------------
+    String CmdAccurateFacing::doGet(const void* target) const
+    {
+        return StringConverter::toString(
+            static_cast<const BillboardParticleRenderer*>(target)->getUseAccurateFacing() );
+    }
+    void CmdAccurateFacing::doSet(void* target, const String& val)
+    {
+        static_cast<BillboardParticleRenderer*>(target)->setUseAccurateFacing(
+            StringConverter::parseBool(val));
+    }
 
 }
 
+
