@@ -31,12 +31,36 @@ THE SOFTWARE.
 
 namespace Ogre
 {
+    /// Command object for setting macro defines
+    class CmdPreprocessorDefines : public ParamCommand
+    {
+    public:
+        //-----------------------------------------------------------------------
+        String doGet(const void* target) const
+        {
+            return static_cast<const HighLevelGpuProgram*>(target)->getPreprocessorDefines();
+        }
+        void doSet(void* target, const String& val)
+        {
+            static_cast<HighLevelGpuProgram*>(target)->setPreprocessorDefines(val);
+        }
+    };
+    static CmdPreprocessorDefines msCmdPreprocessorDefines;
+
+    void HighLevelGpuProgram::setupBaseParamDictionary()
+    {
+        GpuProgram::setupBaseParamDictionary();
+        ParamDictionary* dict = getParamDictionary();
+
+        dict->addParameter(ParameterDef("preprocessor_defines", "", PT_STRING), &msCmdPreprocessorDefines);
+    }
+
     //---------------------------------------------------------------------------
     HighLevelGpuProgram::HighLevelGpuProgram(ResourceManager* creator, 
         const String& name, ResourceHandle handle, const String& group, 
         bool isManual, ManualResourceLoader* loader)
         : GpuProgram(creator, name, handle, group, isManual, loader), 
-        mHighLevelLoaded(false), mAssemblerProgram(), mConstantDefsBuilt(false)
+        mHighLevelLoaded(false), mConstantDefsBuilt(false), mAssemblerProgram()
     {
     }
     //---------------------------------------------------------------------------
@@ -50,7 +74,7 @@ namespace Ogre
             // create low-level implementation
             createLowLevelImpl();
             // load constructed assembler program (if it exists)
-            if (mAssemblerProgram && mAssemblerProgram.get() != this)
+            if (mAssemblerProgram)
             {
                 mAssemblerProgram->load();
             }
@@ -60,7 +84,7 @@ namespace Ogre
     //---------------------------------------------------------------------------
     void HighLevelGpuProgram::unloadImpl()
     {   
-        if (mAssemblerProgram && mAssemblerProgram.get() != this)
+        if (mAssemblerProgram)
         {
             mAssemblerProgram->getCreator()->remove(mAssemblerProgram);
             mAssemblerProgram.reset();
@@ -86,6 +110,7 @@ namespace Ogre
         // Only populate named parameters if we can support this program
         if (this->isSupported())
         {
+            safePrepare(); // loads source
             loadHighLevel();
             // Errors during load may have prevented compile
             if (this->isSupported())
@@ -100,13 +125,10 @@ namespace Ogre
     }
     size_t HighLevelGpuProgram::calculateSize(void) const
     {
-        size_t memSize = 0;
-        memSize += sizeof(bool);
-        if(mAssemblerProgram && (mAssemblerProgram.get() != this) )
+        size_t memSize = GpuProgram::calculateSize();
+        if(mAssemblerProgram)
             memSize += mAssemblerProgram->calculateSize();
 
-        memSize += GpuProgram::calculateSize();
-
         return memSize;
     }
 
@@ -154,7 +176,9 @@ namespace Ogre
                     defines[pos] = '\0';
                     // No definition part, define as "1"
                     ++pos;
-                    ret.push_back({&defines[macro_name_start], "1"});
+
+                    if(defines[macro_name_start] != '\0') // e.g ",DEFINE" or "DEFINE,"
+                        ret.push_back({&defines[macro_name_start], "1"});
                 }
             }
             else
@@ -169,45 +193,36 @@ namespace Ogre
         return ret;
     }
 
+    String HighLevelGpuProgram::appendBuiltinDefines(String defines)
+    {
+        if(!defines.empty()) defines += ",";
+
+        auto renderSystem = Root::getSingleton().getRenderSystem();
+
+        // OGRE_HLSL, OGRE_GLSL etc.
+        String tmp = getLanguage();
+        StringUtil::toUpperCase(tmp);
+        auto ver = renderSystem ? renderSystem->getNativeShadingLanguageVersion() : 0;
+        defines += StringUtil::format("OGRE_%s=%d", tmp.c_str(), ver);
+
+        // OGRE_VERTEX_SHADER, OGRE_FRAGMENT_SHADER
+        tmp = GpuProgram::getProgramTypeName(getType());
+        StringUtil::toUpperCase(tmp);
+        defines += ",OGRE_"+tmp+"_SHADER";
+
+        if(renderSystem && renderSystem->isReverseDepthBufferEnabled())
+            defines += ",OGRE_REVERSED_Z";
+
+        return defines;
+    }
+
     //---------------------------------------------------------------------------
     void HighLevelGpuProgram::loadHighLevel(void)
     {
         if (!mHighLevelLoaded)
         {
-            try 
-            {
-                loadHighLevelImpl();
-                mHighLevelLoaded = true;
-                if (mDefaultParams)
-                {
-                    // Keep a reference to old ones to copy
-                    GpuProgramParametersSharedPtr savedParams = mDefaultParams;
-                    // reset params to stop them being referenced in the next create
-                    mDefaultParams.reset();
-
-                    // Create new params
-                    mDefaultParams = createParameters();
-
-                    // Copy old (matching) values across
-                    // Don't use copyConstantsFrom since program may be different
-                    mDefaultParams->copyMatchingNamedConstantsFrom(*savedParams.get());
-
-                }
-
-            }
-            catch (const RuntimeAssertionException&)
-            {
-                throw;
-            }
-            catch (const Exception& e)
-            {
-                // will already have been logged
-                LogManager::getSingleton().stream(LML_CRITICAL)
-                    << "High-level program '" << mName << "' is not supported: "
-                    << e.getDescription();
-
-                mCompileError = true;
-            }
+            GpuProgram::loadImpl();
+            mHighLevelLoaded = !mCompileError;
         }
     }
     //---------------------------------------------------------------------------
@@ -222,26 +237,9 @@ namespace Ogre
 
             mHighLevelLoaded = false;
         }
-    }
-    //---------------------------------------------------------------------------
-    void HighLevelGpuProgram::loadHighLevelImpl(void)
-    {
-        if (mLoadFromFile)
-        {
-            // find & load source code
-            DataStreamPtr stream = 
-                ResourceGroupManager::getSingleton().openResource(
-                    mFilename, mGroup, this);
-
-            mSource = stream->getAsString();
-        }
-
-        loadFromSource();
-
-
     }
     //---------------------------------------------------------------------
-    const GpuNamedConstants& HighLevelGpuProgram::getConstantDefinitions() const
+    const GpuNamedConstants& HighLevelGpuProgram::getConstantDefinitions()
     {
         if (!mConstantDefsBuilt)
         {
@@ -257,12 +255,11 @@ namespace Ogre
         getConstantDefinitions();
         params->_setNamedConstants(mConstantDefs);
         // also set logical / physical maps for programs which use this
-        params->_setLogicalIndexes(mFloatLogicalToPhysical, mDoubleLogicalToPhysical,
-                                   mIntLogicalToPhysical);
+        params->_setLogicalIndexes(mLogicalToPhysical);
     }
 
     //-----------------------------------------------------------------------
-    String HighLevelGpuProgram::_resolveIncludes(const String& inSource, Resource* resourceBeingLoaded, const String& fileName)
+    String HighLevelGpuProgram::_resolveIncludes(const String& inSource, Resource* resourceBeingLoaded, const String& fileName, bool supportsFilename)
     {
         String outSource;
         // output will be at least this big
@@ -271,7 +268,6 @@ namespace Ogre
         size_t startMarker = 0;
         size_t i = inSource.find("#include");
 
-        bool supportsFilename = StringUtil::endsWith(fileName, "cg");
         String lineFilename = supportsFilename ? StringUtil::format(" \"%s\"", fileName.c_str()) : " 0";
 
         while (i != String::npos)
@@ -306,10 +302,10 @@ namespace Ogre
             }
 
             // find following newline (or EOF)
-            size_t newLineAfter = inSource.find('\n', afterIncludePos);
+            size_t newLineAfter = std::min(inSource.find('\n', afterIncludePos), inSource.size());
             // find include file string container
-            String endDelimeter = "\"";
-            size_t startIt = inSource.find('\"', afterIncludePos);
+            char endDelimiter = '"';
+            size_t startIt = inSource.find('"', afterIncludePos);
             if (startIt == String::npos || startIt > newLineAfter)
             {
                 // try <>
@@ -322,15 +318,16 @@ namespace Ogre
                 }
                 else
                 {
-                    endDelimeter = ">";
+                    endDelimiter = '>';
                 }
             }
-            size_t endIt = inSource.find(endDelimeter, startIt+1);
+            size_t endIt = inSource.find(endDelimiter, startIt+1);
             if (endIt == String::npos || endIt <= startIt)
             {
-                OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "Badly formed #include directive (expected " + endDelimeter +
-                                                               ") in file " + fileName + ": " +
-                                                               inSource.substr(includePos, newLineAfter - includePos));
+                OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+                            "Badly formed #include directive (expected " + String(1, endDelimiter) +
+                                ") in file " + fileName + ": " +
+                                inSource.substr(includePos, newLineAfter - includePos));
             }
 
             // extract filename
@@ -345,16 +342,8 @@ namespace Ogre
             if (newLineBefore != String::npos && newLineBefore >= startMarker)
                 outSource.append(inSource.substr(startMarker, newLineBefore-startMarker+1));
 
-            size_t lineCount = 0;
-            size_t lineCountPos = 0;
-
-            // Count the line number of #include statement
-            lineCountPos = outSource.find('\n');
-            while(lineCountPos != String::npos)
-            {
-                lineCountPos = outSource.find('\n', lineCountPos+1);
-                lineCount++;
-            }
+            // Count the line number of #include statement, account for new line after the statement
+            size_t lineCount = std::count(inSource.begin(), inSource.begin() + newLineAfter, '\n') + 1;
 
             // use include filename if supported (cg) - else use include line as id (glsl)
             String incLineFilename = supportsFilename ? StringUtil::format(" \"%s\"", filename.c_str()) : StringUtil::format(" %zu", lineCount);
@@ -362,10 +351,11 @@ namespace Ogre
             // Add #line to the start of the included file to correct the line count)
             outSource.append("#line 1 " + incLineFilename + "\n");
 
-            outSource.append(resource->getAsString());
+            // recurse into include
+            outSource.append(_resolveIncludes(resource->getAsString(), resourceBeingLoaded, filename, supportsFilename));
 
             // Add #line to the end of the included file to correct the line count
-            outSource.append("\n#line " + std::to_string(lineCount) + lineFilename + "\n");
+            outSource.append("\n#line " + std::to_string(lineCount) + lineFilename);
 
             startMarker = newLineAfter;
 
