@@ -28,11 +28,6 @@ THE SOFTWARE.
 #include "OgreStableHeaders.h"
 #include "OgreResourceBackgroundQueue.h"
 
-#if OGRE_THREAD_SUPPORT == 3 // resource system is not threadsafe
-#undef OGRE_THREAD_SUPPORT
-#define OGRE_THREAD_SUPPORT 0
-#endif
-
 namespace Ogre {
 
     // Note, no locks are required here anymore because all of the parallelisation
@@ -48,7 +43,43 @@ namespace Ogre {
     {  
         assert( msSingleton );  return ( *msSingleton );  
     }
-    //-----------------------------------------------------------------------   
+    //-----------------------------------------------------------------------
+    /** Enumerates the type of requests */
+    enum RequestType
+    {
+        RT_INITIALISE_GROUP = 0,
+        RT_INITIALISE_ALL_GROUPS = 1,
+        RT_PREPARE_GROUP = 2,
+        RT_PREPARE_RESOURCE = 3,
+        RT_LOAD_GROUP = 4,
+        RT_LOAD_RESOURCE = 5,
+        RT_UNLOAD_GROUP = 6,
+        RT_UNLOAD_RESOURCE = 7
+    };
+    /** Encapsulates a queued request for the background queue */
+    struct ResourceRequest
+    {
+        RequestType type;
+        String resourceName;
+        ResourceHandle resourceHandle;
+        String resourceType;
+        String groupName;
+        bool isManual;
+        ManualResourceLoader* loader;
+        NameValuePairList* loadParams;
+        ResourceBackgroundQueue::Listener* listener;
+        BackgroundProcessResult result;
+    };
+    /// Struct that holds details of queued notifications
+    struct ResourceResponse
+    {
+        ResourceResponse(ResourcePtr r, const ResourceRequest& req)
+            : resource(r), request(req)
+        {}
+
+        ResourcePtr resource;
+        ResourceRequest request;
+    };
     //------------------------------------------------------------------------
     ResourceBackgroundQueue::ResourceBackgroundQueue() : mWorkQueueChannel(0)
     {
@@ -308,7 +339,7 @@ namespace Ogre {
             }
             resreq.result.error = false;
             ResourceResponse resresp(ResourcePtr(), resreq);
-            return OGRE_NEW WorkQueue::Response(req, true, Any(resresp));
+            return OGRE_NEW WorkQueue::Response(req, true, resresp);
         }
 
         ResourceManager* rm = 0;
@@ -381,7 +412,7 @@ namespace Ogre {
 
             // return error response
             ResourceResponse resresp(resource, resreq);
-            return OGRE_NEW WorkQueue::Response(req, false, Any(resresp), e.getFullDescription());
+            return OGRE_NEW WorkQueue::Response(req, false, resresp, e.getFullDescription());
         }
 
 
@@ -393,7 +424,7 @@ namespace Ogre {
         }
         resreq.result.error = false;
         ResourceResponse resresp(resource, resreq);
-        return OGRE_NEW WorkQueue::Response(req, true, Any(resresp));
+        return OGRE_NEW WorkQueue::Response(req, true, resresp);
 
     }
     //------------------------------------------------------------------------
