@@ -4,30 +4,28 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreBspSceneManager.h"
-#include "OgreBspResourceManager.h"
 #include "OgreBspNode.h"
 #include "OgreException.h"
 #include "OgreRenderSystem.h"
@@ -45,7 +43,8 @@ Torus Knot Software Ltd.
 #include "OgreTechnique.h"
 #include "OgrePass.h"
 #include "OgreMaterialManager.h"
-
+#include "OgreCodec.h"
+#include "OgreRoot.h"
 
 #include <fstream>
 
@@ -53,31 +52,22 @@ namespace Ogre {
 
     //-----------------------------------------------------------------------
     BspSceneManager::BspSceneManager(const String& name)
-		: SceneManager(name)
+        : SceneManager(name)
     {
         // Set features for debugging render
         mShowNodeAABs = false;
 
-        // No sky by default
-        mSkyPlaneEnabled = false;
-        mSkyBoxEnabled = false;
-        mSkyDomeEnabled = false;
-
-        mLevel.setNull();
+        mLevel.reset();
 
     }
-	//-----------------------------------------------------------------------
-	const String& BspSceneManager::getTypeName(void) const
-	{
-		return BspSceneManagerFactory::FACTORY_TYPE_NAME;
-	}
     //-----------------------------------------------------------------------
-    BspSceneManager::~BspSceneManager()
+    const String& BspSceneManager::getTypeName(void) const
     {
-        freeMemory();
-        mLevel.setNull();
+        return BspSceneManagerFactory::FACTORY_TYPE_NAME;
     }
     //-----------------------------------------------------------------------
+    BspSceneManager::~BspSceneManager() {}
+    //-----------------------------------------------------------------------
     size_t BspSceneManager::estimateWorldGeometry(const String& filename)
     {
         return BspLevel::calculateLoadingStages(filename);
@@ -85,7 +75,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     size_t BspSceneManager::estimateWorldGeometry(DataStreamPtr& stream, 
-		const String& typeName)
+        const String& typeName)
     {
         return BspLevel::calculateLoadingStages(stream);
         
@@ -93,184 +83,89 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void BspSceneManager::setWorldGeometry(const String& filename)
     {
-        mLevel.setNull();
-        // Check extension is .bsp
-        char extension[6];
-        size_t pos = filename.find_last_of(".");
-		if( pos == String::npos )
-            OGRE_EXCEPT(
-				Exception::ERR_INVALIDPARAMS,
-                "Unable to load world geometry. Invalid extension (must be .bsp).",
-                "BspSceneManager::setWorldGeometry");
-
-        strncpy(extension, filename.substr(pos + 1, filename.length() - pos).c_str(), 5);
-		extension[5] = 0;
-
-        if (stricmp(extension, "bsp"))
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
-			"Unable to load world geometry. Invalid extension (must be .bsp).",
-            "BspSceneManager::setWorldGeometry");
-
-        // Load using resource manager
-        mLevel = BspResourceManager::getSingleton().load(filename, 
-            ResourceGroupManager::getSingleton().getWorldResourceGroupName());
-
-		if (mLevel->isSkyEnabled())
-		{
-			// Quake3 is always aligned with Z upwards
-			Quaternion q;
-			q.FromAngleAxis(Radian(Math::HALF_PI), Vector3::UNIT_X);
-			// Also draw last, and make close to camera (far clip plane is shorter)
-			setSkyDome(true, mLevel->getSkyMaterialName(),
-				mLevel->getSkyCurvature(), 12, 2000, false, q);
-		}
-		else
-		{
-			setSkyDome(false, StringUtil::BLANK);
-		}
-
-        // Init static render operation
-        mRenderOp.vertexData = mLevel->mVertexData;
-        // index data is per-frame
-        mRenderOp.indexData = OGRE_NEW IndexData();
-        mRenderOp.indexData->indexStart = 0;
-        mRenderOp.indexData->indexCount = 0;
-        // Create enough index space to render whole level
-        mRenderOp.indexData->indexBuffer = HardwareBufferManager::getSingleton()
-            .createIndexBuffer(
-                HardwareIndexBuffer::IT_32BIT, // always 32-bit
-                mLevel->mNumIndexes, 
-                HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY_DISCARDABLE, false);
-
-        mRenderOp.operationType = RenderOperation::OT_TRIANGLE_LIST;
-        mRenderOp.useIndexes = true;
-
-
+        auto stream = Root::openFileStream(
+            filename, ResourceGroupManager::getSingleton().getWorldResourceGroupName());
+        setWorldGeometry(stream);
     }
     //-----------------------------------------------------------------------
     void BspSceneManager::setWorldGeometry(DataStreamPtr& stream, 
-		const String& typeName)
+        const String& typeName)
     {
-        mLevel.setNull();
+        Codec::getCodec("bsp")->decode(stream, getRootSceneNode());
+    }
 
-        // Load using resource manager
-        mLevel = BspResourceManager::getSingleton().load(stream, 
-			ResourceGroupManager::getSingleton().getWorldResourceGroupName());
-
-		if (mLevel->isSkyEnabled())
-		{
-			// Quake3 is always aligned with Z upwards
-			Quaternion q;
-			q.FromAngleAxis(Radian(Math::HALF_PI), Vector3::UNIT_X);
-			// Also draw last, and make close to camera (far clip plane is shorter)
-			setSkyDome(true, mLevel->getSkyMaterialName(),
-				mLevel->getSkyCurvature(), 12, 2000, false, q);
-		}
-		else
-		{
-			setSkyDome(false, StringUtil::BLANK);
-		}
-
-        // Init static render operation
-        mRenderOp.vertexData = mLevel->mVertexData;
-        // index data is per-frame
-        mRenderOp.indexData = OGRE_NEW IndexData();
-        mRenderOp.indexData->indexStart = 0;
-        mRenderOp.indexData->indexCount = 0;
-        // Create enough index space to render whole level
-        mRenderOp.indexData->indexBuffer = HardwareBufferManager::getSingleton()
-            .createIndexBuffer(
-                HardwareIndexBuffer::IT_32BIT, // always 32-bit
-                mLevel->mNumIndexes, 
-                HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY_DISCARDABLE, false);
-
-        mRenderOp.operationType = RenderOperation::OT_TRIANGLE_LIST;
-        mRenderOp.useIndexes = true;
+    void BspSceneManager::setLevel(const BspLevelPtr& level)
+    {
+        mLevel = level;
 
+        if(!mLevel)
+            return;
 
+        if (mLevel->isSkyEnabled())
+        {
+            // Quake3 is always aligned with Z upwards
+            Quaternion q;
+            q.FromAngleAxis(Radian(Math::HALF_PI), Vector3::UNIT_X);
+            // Also draw last, and make close to camera (far clip plane is shorter)
+            setSkyDome(true, mLevel->getSkyMaterialName(),
+                mLevel->getSkyCurvature(), 12, 2000, false, q);
+        }
+        else
+        {
+            setSkyDome(false, BLANKSTRING);
+        }
     }
     //-----------------------------------------------------------------------
     void BspSceneManager::_findVisibleObjects(Camera* cam, 
-		VisibleObjectsBoundsInfo* visibleBounds, bool onlyShadowCasters)
+        VisibleObjectsBoundsInfo* visibleBounds, bool onlyShadowCasters)
     {
         // Clear unique list of movables for this frame
         mMovablesForRendering.clear();
 
-		// Assemble an AAB on the fly which contains the scene elements visible
-		// by the camera.
-		CamVisibleObjectsMap::iterator findIt = mCamVisibleObjectsMap.find( cam );
+        // Assemble an AAB on the fly which contains the scene elements visible
+        // by the camera.
+        CamVisibleObjectsMap::iterator findIt = mCamVisibleObjectsMap.find( cam );
 
         // Walk the tree, tag static geometry, return camera's node (for info only)
         // Movables are now added to the render queue in processVisibleLeaf
         walkTree(cam, &(findIt->second), onlyShadowCasters);
     }
-	//---------------------------------------------------------------------
-	bool BspSceneManager::fireRenderQueueEnded(uint8 id, const String& invocation)
-	{
-		bool repeat = SceneManager::fireRenderQueueEnded(id, invocation);
-		// Trigger level render just after skies
-		// can't trigger on mWorldGeometryRenderQueue because we're not queueing there
-		if (id == RENDER_QUEUE_SKIES_EARLY)
-		{
-			renderStaticGeometry();
-		}
-		return repeat;
+    //---------------------------------------------------------------------
+    bool BspSceneManager::fireRenderQueueEnded(uint8 id, const String& invocation)
+    {
+        bool repeat = SceneManager::fireRenderQueueEnded(id, invocation);
+        // Trigger level render just after skies
+        // can't trigger on mWorldGeometryRenderQueue because we're not queueing there
+        if (id == RENDER_QUEUE_SKIES_EARLY)
+        {
+            renderStaticGeometry();
+        }
+        return repeat;
 
-	}
+    }
     //-----------------------------------------------------------------------
     void BspSceneManager::renderStaticGeometry(void)
     {
-		// Check we should be rendering
-		if (!isRenderQueueToBeProcessed(mWorldGeometryRenderQueue))
-			return;
-
-        // Cache vertex/face data first
-        std::vector<StaticFaceGroup*>::const_iterator faceGrpi;
-        static RenderOperation patchOp;
-        
-        // no world transform required
-        mDestRenderSystem->_setWorldMatrix(Matrix4::IDENTITY);
-        // Set view / proj
-        mDestRenderSystem->_setViewMatrix(mCachedViewMatrix);
-        mDestRenderSystem->_setProjectionMatrix(mCameraInProgress->getProjectionMatrixRS());
+        // Check we should be rendering
+        if (!isRenderQueueToBeProcessed(mWorldGeometryRenderQueue))
+            return;
 
         // For each material in turn, cache rendering data & render
         MaterialFaceGroupMap::const_iterator mati;
-
         for (mati = mMatFaceGroupMap.begin(); mati != mMatFaceGroupMap.end(); ++mati)
         {
             // Get Material
             Material* thisMaterial = mati->first;
-
-            // Empty existing cache
-            mRenderOp.indexData->indexCount = 0;
-            // lock index buffer ready to receive data
-            unsigned int* pIdx = static_cast<unsigned int*>(
-                mRenderOp.indexData->indexBuffer->lock(HardwareBuffer::HBL_DISCARD));
-
-            for (faceGrpi = mati->second.begin(); faceGrpi != mati->second.end(); ++faceGrpi)
-            {
-                // Cache each
-                unsigned int numelems = cacheGeometry(pIdx, *faceGrpi);
-                mRenderOp.indexData->indexCount += numelems;
-                pIdx += numelems;
-            }
-            // Unlock the buffer
-            mRenderOp.indexData->indexBuffer->unlock();
+            thisMaterial->touch();
 
             // Skip if no faces to process (we're not doing flare types yet)
-            if (mRenderOp.indexData->indexCount == 0)
+            if (!mLevel->cacheGeometry(mati->second))
                 continue;
 
-            Technique::PassIterator pit = thisMaterial->getTechnique(0)->getPassIterator();
-
-            while (pit.hasMoreElements())
+            const Technique::Passes& passes = thisMaterial->getBestTechnique ()->getPasses();
+            for (size_t p = 0; p < passes.size(); p++)
             {
-                _setPass(pit.getNext());
-
-                mDestRenderSystem->_render(mRenderOp);
-
-
+                _injectRenderWithPass(passes[p], mLevel.get());
             } 
 
 
@@ -293,9 +188,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     BspNode* BspSceneManager::walkTree(Camera* camera, 
-		VisibleObjectsBoundsInfo *visibleBounds, bool onlyShadowCasters)
+        VisibleObjectsBoundsInfo *visibleBounds, bool onlyShadowCasters)
     {
-		if (mLevel.isNull()) return 0;
+        if (!mLevel) return 0;
 
         // Locate the leaf node where the camera is located
         BspNode* cameraNode = mLevel->findLeaf(camera->getDerivedPosition());
@@ -365,7 +260,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     void BspSceneManager::processVisibleLeaf(BspNode* leaf, Camera* cam, 
-		VisibleObjectsBoundsInfo* visibleBounds, bool onlyShadowCasters)
+        VisibleObjectsBoundsInfo* visibleBounds, bool onlyShadowCasters)
     {
         MaterialPtr pMat;
         // Skip world geometry if we're only supposed to process shadow casters
@@ -384,8 +279,8 @@ namespace Ogre {
                     continue;
                 StaticFaceGroup* faceGroup = mLevel->mFaceGroups + realIndex;
                 // Get Material pointer by handle
-                pMat = MaterialManager::getSingleton().getByHandle(faceGroup->materialHandle);
-                assert (!pMat.isNull());
+                pMat = static_pointer_cast<Material>(MaterialManager::getSingleton().getByHandle(faceGroup->materialHandle));
+                assert (pMat);
                 // Check normal (manual culling)
                 ManualCullingMode cullMode = pMat->getTechnique(0)->getPass(0)->getManualCullingMode();
                 if (cullMode != MANUAL_CULL_NONE)
@@ -398,9 +293,7 @@ namespace Ogre {
                 mFaceGroupSet.insert(realIndex);
                 // Try to insert, will find existing if already there
                 std::pair<MaterialFaceGroupMap::iterator, bool> matgrpi;
-                matgrpi = mMatFaceGroupMap.insert(
-                    MaterialFaceGroupMap::value_type(pMat.getPointer(), std::vector<StaticFaceGroup*>())
-                    );
+                matgrpi = mMatFaceGroupMap.emplace(pMat.get(), std::vector<StaticFaceGroup*>());
                 // Whatever happened, matgrpi.first is map iterator
                 // Need to get second part of that to get vector
                 matgrpi.first->second.push_back(faceGroup);
@@ -423,70 +316,12 @@ namespace Ogre {
                 // It hasn't been seen yet
                 MovableObject *mov = const_cast<MovableObject*>(*oi); // hacky
 
-				getRenderQueue()->processVisibleObject(mov, cam, onlyShadowCasters, visibleBounds);
+                getRenderQueue()->processVisibleObject(mov, cam, onlyShadowCasters, visibleBounds);
 
             }
         }
 
 
-    }
-    //-----------------------------------------------------------------------
-    unsigned int BspSceneManager::cacheGeometry(unsigned int* pIndexes, 
-        const StaticFaceGroup* faceGroup)
-    {
-        // Skip sky always
-        if (faceGroup->isSky)
-            return 0;
-
-        size_t idxStart, numIdx, vertexStart;
-
-        if (faceGroup->fType == FGT_FACE_LIST)
-        {
-            idxStart = faceGroup->elementStart;
-            numIdx = faceGroup->numElements;
-            vertexStart = faceGroup->vertexStart;
-        }
-        else if (faceGroup->fType == FGT_PATCH)
-        {
-
-            idxStart = faceGroup->patchSurf->getIndexOffset();
-            numIdx = faceGroup->patchSurf->getCurrentIndexCount();
-            vertexStart = faceGroup->patchSurf->getVertexOffset();
-        }
-		else
-		{
-			// Unsupported face type
-			return 0;
-		}
-
-
-        // Copy index data
-        unsigned int* pSrc = static_cast<unsigned int*>(
-            mLevel->mIndexes->lock(
-                idxStart * sizeof(unsigned int),
-                numIdx * sizeof(unsigned int), 
-                HardwareBuffer::HBL_READ_ONLY));
-        // Offset the indexes here
-        // we have to do this now rather than up-front because the 
-        // indexes are sometimes reused to address different vertex chunks
-        for (size_t elem = 0; elem < numIdx; ++elem)
-        {
-            *pIndexes++ = *pSrc++ + vertexStart;
-        }
-        mLevel->mIndexes->unlock();
-
-        // return number of elements
-        return static_cast<unsigned int>(numIdx);
-
-
-    }
-
-    //-----------------------------------------------------------------------
-    void BspSceneManager::freeMemory(void)
-    {
-        // no need to delete index buffer, will be handled by shared pointer
-        OGRE_DELETE mRenderOp.indexData;
-		mRenderOp.indexData = 0;
     }
     //-----------------------------------------------------------------------
     void BspSceneManager::showNodeBoxes(bool show)
@@ -568,7 +403,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     ViewPoint BspSceneManager::getSuggestedViewpoint(bool random)
     {
-        if (mLevel.isNull() || mLevel->mPlayerStarts.size() == 0)
+        if (!mLevel || mLevel->mPlayerStarts.empty())
         {
             // No level, use default
             return SceneManager::getSuggestedViewpoint(random);
@@ -603,27 +438,26 @@ namespace Ogre {
     void BspSceneManager::_notifyObjectMoved(const MovableObject* mov, 
         const Vector3& pos)
     {
-		if (!mLevel.isNull())
-		{
-			mLevel->_notifyObjectMoved(mov, pos);
-		}
+        if (mLevel)
+        {
+            mLevel->_notifyObjectMoved(mov, pos);
+        }
     }
     //-----------------------------------------------------------------------
-	void BspSceneManager::_notifyObjectDetached(const MovableObject* mov)
-	{
-		if (!mLevel.isNull())
-		{
-			mLevel->_notifyObjectDetached(mov);
-		}
-	}
-	//-----------------------------------------------------------------------
-	void BspSceneManager::clearScene(void)
-	{
-		SceneManager::clearScene();
-		freeMemory();
-		// Clear level
-		mLevel.setNull();
-	}
+    void BspSceneManager::_notifyObjectDetached(const MovableObject* mov)
+    {
+        if (mLevel)
+        {
+            mLevel->_notifyObjectDetached(mov);
+        }
+    }
+    //-----------------------------------------------------------------------
+    void BspSceneManager::clearScene(void)
+    {
+        SceneManager::clearScene();
+        // Clear level
+        mLevel.reset();
+    }
     //-----------------------------------------------------------------------
     /*
     AxisAlignedBoxSceneQuery* BspSceneManager::
@@ -642,7 +476,7 @@ namespace Ogre {
     */
     //-----------------------------------------------------------------------
     RaySceneQuery* BspSceneManager::
-    createRayQuery(const Ray& ray, unsigned long mask)
+    createRayQuery(const Ray& ray, uint32 mask)
     {
         BspRaySceneQuery* q = OGRE_NEW BspRaySceneQuery(this);
         q->setRay(ray);
@@ -651,7 +485,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     IntersectionSceneQuery* BspSceneManager::
-    createIntersectionQuery(unsigned long mask)
+    createIntersectionQuery(uint32 mask)
     {
         BspIntersectionSceneQuery* q = OGRE_NEW BspIntersectionSceneQuery(this);
         q->setQueryMask(mask);
@@ -674,7 +508,7 @@ namespace Ogre {
         overlap 2 leaves?
         */
         const BspLevelPtr& lvl = ((BspSceneManager*)mParentSceneMgr)->getLevel();
-		if (lvl.isNull()) return;
+        if (!lvl) return;
 
         BspNode* leaf = lvl->getLeafStart();
         int numLeaves = lvl->getNumLeaves();
@@ -692,8 +526,8 @@ namespace Ogre {
                 const MovableObject* aObj = *a;
                 // Skip this object if collision not enabled
                 if (!(aObj->getQueryFlags() & mQueryMask) ||
-					!(aObj->getTypeFlags() & mQueryTypeMask) ||
-					!aObj->isInScene())
+                    !(aObj->getTypeFlags() & mQueryTypeMask) ||
+                    !aObj->isInScene())
                     continue;
 
                 if (oi < (numObjects-1))
@@ -705,8 +539,8 @@ namespace Ogre {
                         const MovableObject* bObj = *b;
                         // Apply mask to b (both must pass)
                         if ((bObj->getQueryFlags() & mQueryMask) && 
-							(bObj->getTypeFlags() & mQueryTypeMask) && 
-							bObj->isInScene())
+                            (bObj->getTypeFlags() & mQueryTypeMask) && 
+                            bObj->isInScene())
                         {
                             const AxisAlignedBox& box1 = aObj->getWorldBoundingBox();
                             const AxisAlignedBox& box2 = bObj->getWorldBoundingBox();
@@ -715,7 +549,7 @@ namespace Ogre {
                             {
                                 if (!listener->queryResult(const_cast<MovableObject*>(aObj), 
                                     const_cast<MovableObject*>(bObj)))
-									return; 
+                                    return; 
                             }
                         }
                     }
@@ -731,7 +565,7 @@ namespace Ogre {
 
                     for (bi = brushes.begin(); bi != biend; ++bi)
                     {
-                        std::list<Plane>::const_iterator planeit, planeitend;
+                        std::vector<Plane>::const_iterator planeit, planeitend;
                         planeitend = (*bi)->planes.end();
                         bool brushIntersect = true; // Assume intersecting for now
 
@@ -751,7 +585,7 @@ namespace Ogre {
                             assert((*bi)->fragment.fragmentType == SceneQuery::WFT_PLANE_BOUNDED_REGION);
                             if (!listener->queryResult(const_cast<MovableObject*>(aObj), 
                                     const_cast<WorldFragment*>(&((*bi)->fragment))))
-								return; 
+                                return; 
                         }
 
                     }
@@ -779,13 +613,13 @@ namespace Ogre {
     void BspRaySceneQuery::execute(RaySceneQueryListener* listener)
     {
         clearTemporaries();
-		BspLevelPtr lvl = static_cast<BspSceneManager*>(mParentSceneMgr)->getLevel();
-		if (!lvl.isNull())
-		{
-			processNode(
-				lvl->getRootNode(), 
-				mRay, listener);
-		}
+        BspLevelPtr lvl = static_cast<BspSceneManager*>(mParentSceneMgr)->getLevel();
+        if (lvl)
+        {
+            processNode(
+                lvl->getRootNode(), 
+                mRay, listener);
+        }
     }
     //-----------------------------------------------------------------------
     BspRaySceneQuery::~BspRaySceneQuery()
@@ -869,7 +703,7 @@ namespace Ogre {
             MovableObject* obj = const_cast<MovableObject*>(*i);
             // Skip this object if not enabled
             if(!(obj->getQueryFlags() & mQueryMask) ||
-				!((obj->getTypeFlags() & mQueryTypeMask)))
+                !((obj->getTypeFlags() & mQueryTypeMask)))
                 continue;
 
             // check we haven't reported this one already
@@ -885,7 +719,7 @@ namespace Ogre {
             if(result.first && result.second <= maxDistance)
             {
                 if (!listener->queryResult(obj, result.second + traceDistance))
-					return false;
+                    return false;
             }
         }
 
@@ -918,7 +752,7 @@ namespace Ogre {
                         // save this so we can clean up later
                         mSingleIntersections.push_back(wf);
                         if (!listener->queryResult(wf, result.second + traceDistance))
-							return false;
+                            return false;
                     }
                     else if (mWorldFragmentType ==  SceneQuery::WFT_PLANE_BOUNDED_REGION)
                     {
@@ -926,7 +760,7 @@ namespace Ogre {
                         assert((*bi)->fragment.fragmentType == SceneQuery::WFT_PLANE_BOUNDED_REGION);
                         if (!listener->queryResult(const_cast<WorldFragment*>(&(brush->fragment)), 
                             result.second + traceDistance))
-							return false; 
+                            return false; 
 
                     }
                 }
@@ -941,27 +775,25 @@ namespace Ogre {
 
     } 
     //-----------------------------------------------------------------------
-	//-----------------------------------------------------------------------
-	const String BspSceneManagerFactory::FACTORY_TYPE_NAME = "BspSceneManager";
-	//-----------------------------------------------------------------------
-	void BspSceneManagerFactory::initMetaData(void) const
-	{
-		mMetaData.typeName = FACTORY_TYPE_NAME;
-		mMetaData.description = "Scene manager for loading Quake3 .bsp files.";
-		mMetaData.sceneTypeMask = ST_INTERIOR;
-		mMetaData.worldGeometrySupported = true;
-	}
-	//-----------------------------------------------------------------------
-	SceneManager* BspSceneManagerFactory::createInstance(
-		const String& instanceName)
-	{
-		return OGRE_NEW BspSceneManager(instanceName);
-	}
-	//-----------------------------------------------------------------------
-	void BspSceneManagerFactory::destroyInstance(SceneManager* instance)
-	{
-		OGRE_DELETE instance;
-	}
+    //-----------------------------------------------------------------------
+    const String BspSceneManagerFactory::FACTORY_TYPE_NAME = "BspSceneManager";
+    //-----------------------------------------------------------------------
+    void BspSceneManagerFactory::initMetaData(void) const
+    {
+        mMetaData.typeName = FACTORY_TYPE_NAME;
+        mMetaData.worldGeometrySupported = true;
+    }
+    //-----------------------------------------------------------------------
+    SceneManager* BspSceneManagerFactory::createInstance(
+        const String& instanceName)
+    {
+        return OGRE_NEW BspSceneManager(instanceName);
+    }
+    //-----------------------------------------------------------------------
+    void BspSceneManagerFactory::destroyInstance(SceneManager* instance)
+    {
+        OGRE_DELETE instance;
+    }
 
 }
 
