@@ -55,7 +55,9 @@ namespace Ogre
     mShowPortals(false),
     mZoneFactoryManager(0),
     mActiveCameraZone(0)
-    { }
+    {
+        addShadowTextureListener(this);
+    }
 
     PCZSceneManager::~PCZSceneManager()
     {
@@ -445,8 +447,6 @@ namespace Ogre
         // Clear animations
         destroyAllAnimations();
 
-        mSkyRenderer.clear();
-
         // Clear render queue, empty completely
         if (mRenderQueue)
             mRenderQueue->clear(true);
@@ -489,17 +489,17 @@ namespace Ogre
     /* enable/disable sky rendering */
     void PCZSceneManager::enableSky(bool onoff)
     {
-        if (mSkyRenderer.mSkyBoxNode)
+        if (mSkyBox.mSceneNode)
         {
-            mSkyRenderer.mSkyBoxEnabled = onoff;
+            mSkyBox.setEnabled(onoff);
         }
-        else if (mSkyRenderer.mSkyDomeNode)
+        else if (mSkyDome.mSceneNode)
         {
-            mSkyRenderer.mSkyDomeEnabled = onoff;
+            mSkyDome.setEnabled(onoff);
         }
-        else if (mSkyRenderer.mSkyPlaneNode)
+        else if (mSkyPlane.mSceneNode)
         {
-            mSkyRenderer.mSkyPlaneEnabled = onoff;
+            mSkyPlane.setEnabled(onoff);
         }
     }
 
@@ -511,22 +511,22 @@ namespace Ogre
             // if no zone specified, use default zone
             zone = mDefaultZone;
         }
-        if (mSkyRenderer.mSkyBoxNode)
+        if (auto node = (PCZSceneNode*)mSkyBox.mSceneNode)
         {
-            ((PCZSceneNode*)mSkyRenderer.mSkyBoxNode)->setHomeZone(zone);
-            ((PCZSceneNode*)mSkyRenderer.mSkyBoxNode)->anchorToHomeZone(zone);
+            node->setHomeZone(zone);
+            node->anchorToHomeZone(zone);
             zone->setHasSky(true);
         }
-        if (mSkyRenderer.mSkyDomeNode)
+        if (auto node = (PCZSceneNode*)mSkyDome.mSceneNode)
         {
-            ((PCZSceneNode*)mSkyRenderer.mSkyDomeNode)->setHomeZone(zone);
-            ((PCZSceneNode*)mSkyRenderer.mSkyDomeNode)->anchorToHomeZone(zone);
+            node->setHomeZone(zone);
+            node->anchorToHomeZone(zone);
             zone->setHasSky(true);
         }
-        if (mSkyRenderer.mSkyPlaneNode)
+        if (auto node = (PCZSceneNode*)mSkyPlane.mSceneNode)
         {
-            ((PCZSceneNode*)mSkyRenderer.mSkyPlaneNode)->setHomeZone(zone);
-            ((PCZSceneNode*)mSkyRenderer.mSkyPlaneNode)->anchorToHomeZone(zone);
+            node->setHomeZone(zone);
+            node->anchorToHomeZone(zone);
             zone->setHasSky(true);
         }
         
@@ -1041,30 +1041,7 @@ namespace Ogre
                 }
             }
 
-            // Sort the lights if using texture shadows, since the first 'n' will be
-            // used to generate shadow textures and we should pick the most appropriate
-            if (isShadowTechniqueTextureBased())
-            {
-                // Allow a ShadowListener to override light sorting
-                // Reverse iterate so last takes precedence
-                bool overridden = false;
-                for (ListenerList::reverse_iterator ri = mListeners.rbegin();
-                    ri != mListeners.rend(); ++ri)
-                {
-                    overridden = (*ri)->sortLightsAffectingFrustum(mLightsAffectingFrustum);
-                    if (overridden)
-                        break;
-                }
-                if (!overridden)
-                {
-                    // default sort (stable to preserve directional light ordering
-                    std::stable_sort(
-                        mLightsAffectingFrustum.begin(), mLightsAffectingFrustum.end(), 
-                        lightsForShadowTextureLess());
-                }
-                
-            }
-
+            mShadowRenderer.sortLightsAffectingFrustum(mLightsAffectingFrustum);
             // Use swap instead of copy operator for efficiently
             mCachedLightInfos.swap(mTestLightInfos);
 
@@ -1084,25 +1061,12 @@ namespace Ogre
         size_t count = mShadowRenderer.mShadowTextureCameras.size();
         for (size_t i = 0; i < count; ++i)
         {
-            PCZSceneNode* node = (PCZSceneNode*)mSceneRoot->createChildSceneNode(
-                    mShadowRenderer.mShadowTextureCameras[i]->getName());
-            node->attachObject(mShadowRenderer.mShadowTextureCameras[i]);
+            auto node = (PCZSceneNode*)mShadowRenderer.mShadowTextureCameras[i]->getParentSceneNode();
             addPCZSceneNode(node, mDefaultZone);
         }
     }
     //---------------------------------------------------------------------
-    void PCZSceneManager::destroyShadowTextures(void)
-    {
-        size_t count = mShadowRenderer.mShadowTextureCameras.size();
-        for (size_t i = 0; i < count; ++i)
-        {
-            SceneNode* node = mShadowRenderer.mShadowTextureCameras[i]->getParentSceneNode();
-            mSceneRoot->removeAndDestroyChild(node);
-        }
-        SceneManager::destroyShadowTextures();
-    }
-    //---------------------------------------------------------------------
-    void PCZSceneManager::fireShadowTexturesPreCaster(Light* light, Camera* camera, size_t iteration)
+    void PCZSceneManager::shadowTextureCasterPreViewProj(Light* light, Camera* camera, size_t iteration)
     {
         PCZSceneNode* camNode = (PCZSceneNode*)camera->getParentSceneNode();
 
@@ -1119,8 +1083,6 @@ namespace Ogre
             if (camNode->getHomeZone() != lightZone)
                 addPCZSceneNode(camNode, lightZone);
         }
-
-        SceneManager::fireShadowTexturesPreCaster(light, camera, iteration);
     }
 
     /* Attempt to automatically connect unconnected portals to proper target zones 
