@@ -69,12 +69,6 @@ namespace Ogre
     private:
         /// Direct3D
         IDirect3D9*  mD3D;
-        // TODO: remove following fields, use values directly from mOptions map as other render systems does
-        size_t mFSAASamples;
-        String mFSAAHint;
-        bool mVSync; 
-        unsigned int mVSyncInterval;
-        unsigned int mBackBufferCount; // -1 means 2 for vsync and 1 for no vsync
 
         /// instance
         HINSTANCE mhInstance;
@@ -98,6 +92,8 @@ namespace Ogre
 
         bool mEnableFixedPipeline;
 
+        bool mAutoHardwareBufferManagement;
+
         /// structure holding texture unit settings for every stage
         struct sD3DTextureStageDesc
         {
@@ -161,7 +157,7 @@ namespace Ogre
         virtual void initialiseFromRenderSystemCapabilities(RenderSystemCapabilities* caps, RenderTarget* primary);
 
 
-        void convertVertexShaderCaps(RenderSystemCapabilities* rsc) const;
+        void convertVertexShaderCaps(RenderSystemCapabilities* rsc);
         void convertPixelShaderCaps(RenderSystemCapabilities* rsc) const;
         bool checkVertexTextureFormats(D3D9RenderWindow* renderWindow) const;
         void detachRenderTargetImpl(const String& name);
@@ -170,7 +166,7 @@ namespace Ogre
         /// Saved last view matrix
         Matrix4 mViewMatrix;
 
-        D3DXMATRIX mDxViewMat, mDxProjMat, mDxWorldMat;
+        D3DMATRIX mDxViewMat, mDxProjMat, mDxWorldMat;
     
         typedef std::vector<D3D9RenderWindow*> D3D9RenderWindowList;
         // List of additional windows after the first (swap chains)
@@ -210,10 +206,6 @@ namespace Ogre
         RenderWindow* _createRenderWindow(const String &name, unsigned int width, unsigned int height, 
             bool fullScreen, const NameValuePairList *miscParams = 0);
         
-        /// @copydoc RenderSystem::_createRenderWindows
-        bool _createRenderWindows(const RenderWindowDescriptionList& renderWindowDescriptions, 
-            RenderWindowList& createdWindows);
-
         /// @copydoc RenderSystem::_createDepthBufferFor
         DepthBuffer* _createDepthBufferFor( RenderTarget *renderTarget );
 
@@ -260,22 +252,24 @@ namespace Ogre
         const String& getName() const;
 
         // Low-level overridden members
+        /**
+         Specific options:
+
+        | Key |  Default | Description |
+        |-----|---------------|---------|
+        | Allow DirectX9Ex | No | Use Direct3D 9Ex if possible |
+        | Multi device memory hint | Auto hardware buffers management | Automatically restore hardware buffers on device lost |
+        | Resource Creation Policy | - | See @ref D3D9ResourceCreationPolicy |
+        | Fixed Pipeline Enabled | true | Use fixed function units where possible. Disable to test migration to shader-only pipeline |
+        | Rendering Device | (default) |  |
+        */
         void setConfigOption( const String &name, const String &value );
-        void reinitialise();
         void shutdown();
         void setAmbientLight( float r, float g, float b );
         void setShadingType( ShadeOptions so );
         void setLightingEnabled( bool enabled );
         void destroyRenderTarget(const String& name);
-        VertexElementType getColourVertexElementType() const;
-        void setStencilCheckEnabled(bool enabled);
-        void setStencilBufferParams(CompareFunction func = CMPF_ALWAYS_PASS, 
-            uint32 refValue = 0, uint32 compareMask = 0xFFFFFFFF, uint32 writeMask = 0xFFFFFFFF,
-            StencilOperation stencilFailOp = SOP_KEEP, 
-            StencilOperation depthFailOp = SOP_KEEP,
-            StencilOperation passOp = SOP_KEEP, 
-            bool twoSidedOperation = false,
-            bool readBackAsTexture = false);
+        void setStencilState(const StencilState& state) override;
         void setNormaliseNormals(bool normalise);
 
         // Low-level overridden members, mainly for internal use
@@ -293,7 +287,6 @@ namespace Ogre
         void _setTextureBlendMode( size_t unit, const LayerBlendModeEx& bm );
         void _setTextureAddressingMode(size_t stage, const Sampler::UVWAddressingMode& uvw);
         void _setTextureMatrix( size_t unit, const Matrix4 &xform );
-        void _setSeparateSceneBlending( SceneBlendFactor sourceFactor, SceneBlendFactor destFactor, SceneBlendFactor sourceFactorAlpha, SceneBlendFactor destFactorAlpha, SceneBlendOperation op, SceneBlendOperation alphaOp );
         void _setAlphaRejectSettings( CompareFunction func, unsigned char value, bool alphaToCoverage );
         void _setViewport( Viewport *vp );      
         void _beginFrame();
@@ -303,7 +296,7 @@ namespace Ogre
         void _setCullingMode( CullingMode mode );
         void _setDepthBufferParams( bool depthTest = true, bool depthWrite = true, CompareFunction depthFunction = CMPF_LESS_EQUAL );
         void _setDepthBufferCheckEnabled( bool enabled = true );
-        void _setColourBufferWriteEnabled(bool red, bool green, bool blue, bool alpha);
+        void setColourBlendState(const ColourBlendState& state);
         void _setDepthBufferWriteEnabled(bool enabled = true);
         void _setDepthBufferFunction( CompareFunction func = CMPF_LESS_EQUAL );
         void _setDepthBias(float constantBias, float slopeScaleBias);
@@ -325,10 +318,10 @@ namespace Ogre
         void bindGpuProgramParameters(GpuProgramType gptype, 
             const GpuProgramParametersPtr& params, uint16 variabilityMask);
 
-        void setScissorTest(bool enabled, size_t left = 0, size_t top = 0, size_t right = 800, size_t bottom = 600);
+        void setScissorTest(bool enabled, const Rect& rect = Rect());
         void clearFrameBuffer(unsigned int buffers, 
             const ColourValue& colour = ColourValue::Black, 
-            Real depth = 1.0f, unsigned short stencil = 0);
+            float depth = 1.0f, unsigned short stencil = 0);
         void setClipPlane (ushort index, Real A, Real B, Real C, Real D);
         void enableClipPlane (ushort index, bool enable);
         HardwareOcclusionQuery* createHardwareOcclusionQuery();
@@ -336,15 +329,6 @@ namespace Ogre
         Real getVerticalTexelOffset();
         Real getMinimumDepthInputValue();
         Real getMaximumDepthInputValue();
-        void registerThread();
-        void unregisterThread();
-        void preExtraThreadsStarted();
-        void postExtraThreadsStarted();     
-                
-        /*
-        Returns whether under the current render system buffers marked as TU_STATIC can be locked for update
-        */
-        virtual bool isStaticBufferLockable() const { return !mIsDirectX9Ex; }
 
 		bool IsActiveDeviceLost();
 
@@ -360,6 +344,8 @@ namespace Ogre
         static IDirect3DDevice9* getActiveD3D9Device();
         static IDirect3DDevice9* getActiveD3D9DeviceIfExists();
 
+        uint32 getAdapterNumber();
+
         /** Check which depthStencil formats can be used with a certain pixel format,
             and return the best suited.
         */
