@@ -35,59 +35,17 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     D3D9HLSLProgram::CmdEntryPoint D3D9HLSLProgram::msCmdEntryPoint;
     D3D9HLSLProgram::CmdTarget D3D9HLSLProgram::msCmdTarget;
-    D3D9HLSLProgram::CmdPreprocessorDefines D3D9HLSLProgram::msCmdPreprocessorDefines;
     D3D9HLSLProgram::CmdColumnMajorMatrices D3D9HLSLProgram::msCmdColumnMajorMatrices;
     D3D9HLSLProgram::CmdOptimisation D3D9HLSLProgram::msCmdOptimisation;
     D3D9HLSLProgram::CmdMicrocode D3D9HLSLProgram::msCmdMicrocode;
     D3D9HLSLProgram::CmdAssemblerCode D3D9HLSLProgram::msCmdAssemblerCode;
     D3D9HLSLProgram::CmdBackwardsCompatibility D3D9HLSLProgram::msCmdBackwardsCompatibility;
 
-    class _OgreD3D9Export HLSLIncludeHandler : public ID3DXInclude
-    {
-    public:
-        HLSLIncludeHandler(Resource* sourceProgram) 
-            : mProgram(sourceProgram) {}
-        ~HLSLIncludeHandler() {}
-        
-        STDMETHOD(Open)(D3DXINCLUDE_TYPE IncludeType,
-            LPCSTR pFileName,
-            LPCVOID pParentData,
-            LPCVOID *ppData,
-            UINT *pByteLen
-            )
-        {
-            // find & load source code
-            DataStreamPtr stream = 
-                ResourceGroupManager::getSingleton().openResource(
-                String(pFileName), mProgram->getGroup(), true, mProgram);
-
-            String source = stream->getAsString();
-            // copy into separate c-string
-            // Note - must NOT copy the null terminator, otherwise this will terminate
-            // the entire program string!
-            *pByteLen = static_cast<UINT>(source.length());
-            char* pChar = OGRE_ALLOC_T(char, *pByteLen, MEMCATEGORY_RESOURCE);
-            memcpy(pChar, source.c_str(), *pByteLen);
-            *ppData = pChar;
-
-            return S_OK;
-        }
-
-        STDMETHOD(Close)(LPCVOID pData)
-        {           
-            OGRE_FREE(pData, MEMCATEGORY_RESOURCE);
-            return S_OK;
-        }
-    protected:
-        Resource* mProgram;
-
-
-    };
-
     //-----------------------------------------------------------------------
-    //-----------------------------------------------------------------------
-    void D3D9HLSLProgram::loadFromSource(void)
+    void D3D9HLSLProgram::prepareImpl()
     {
+        HighLevelGpuProgram::prepareImpl();
+
         uint32 hash = _getHash();
         if ( GpuProgramManager::getSingleton().isMicrocodeAvailableInCache(hash) )
         {
@@ -145,27 +103,35 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+    const String& D3D9HLSLProgram::getTarget() const
+    {
+        if(mTarget.empty())
+        {
+            static String vs_2_0 = "vs_2_0", ps_2_0 = "ps_2_0";
+            return mType == GPT_VERTEX_PROGRAM ? vs_2_0 : ps_2_0;
+        }
+
+        return mTarget;
+    }
+
     void D3D9HLSLProgram::compileMicrocode(void)
     {
         // Populate preprocessor defines
         String stringBuffer;
         std::vector<D3DXMACRO> defines;
         const D3DXMACRO* pDefines = 0;
-        if (!mPreprocessorDefines.empty())
+        stringBuffer = appendBuiltinDefines(mPreprocessorDefines);
+
+        for(const auto& def : parseDefines(stringBuffer))
         {
-            stringBuffer = mPreprocessorDefines;
-
-            for(const auto& def : parseDefines(stringBuffer))
-            {
-                defines.push_back({def.first, def.second});
-            }
-
-            // Add NULL terminator
-            defines.push_back({0, 0});
-
-            pDefines = &defines[0];
+            defines.push_back({def.first, def.second});
         }
 
+        // Add NULL terminator
+        defines.push_back({0, 0});
+
+        pDefines = &defines[0];
+
         // Populate compile flags
         DWORD compileFlags = 0;
         if (mColumnMajorMatrices)
@@ -203,8 +169,8 @@ namespace Ogre {
 
         LPD3DXBUFFER errors = 0;
 
-        // include handler
-        HLSLIncludeHandler includeHandler(this);
+        // handle includes
+        mSource = _resolveIncludes(mSource, this, mFilename, true);
 
         LPD3DXCONSTANTTABLE pConstTable;
 
@@ -213,9 +179,9 @@ namespace Ogre {
             mSource.c_str(),
             static_cast<UINT>(mSource.length()),
             pDefines,
-            &includeHandler, 
+            NULL,
             mEntryPoint.c_str(),
-            mTarget.c_str(),
+            getTarget().c_str(),
             compileFlags,
             &mMicroCode,
             &errors,
@@ -231,8 +197,7 @@ namespace Ogre {
                 errors->Release();
             }
 
-            OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, message,
-                "D3D9HLSLProgram::loadFromSource");
+            OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, message);
         }
         else
         {
@@ -245,9 +210,8 @@ namespace Ogre {
 
             if (FAILED(hr))
             {
-                OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-                    "Cannot retrieve constant descriptions from HLSL program.", 
-                    "D3D9HLSLProgram::buildParameterNameMap");
+                OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+                            "Cannot retrieve constant descriptions from HLSL program");
             }
             // Iterate over the constants
             for (unsigned int i = 0; i < desc.Constants; ++i)
@@ -310,14 +274,14 @@ namespace Ogre {
     {
         if (!mCompileError)
         {
-            // Create a low-level program, give it the same name as us
+            // Create a low-level program
             mAssemblerProgram = 
-                GpuProgramManager::getSingleton().createProgramFromString(
-                    mName, 
+                GpuProgramManager::getSingleton().createProgram(
+                    mName+"/Delegate",
                     mGroup,
-                    "",// dummy source, since we'll be using microcode
-                    mType, 
-                    mTarget);
+                    getTarget(),
+                    mType);
+            mAssemblerProgram->setSource("");
             static_cast<D3D9GpuProgram*>(mAssemblerProgram.get())->setExternalMicrocode(mMicroCode);
         }
 
@@ -330,38 +294,23 @@ namespace Ogre {
         SAFE_RELEASE(mMicroCode);
     }
     //-----------------------------------------------------------------------
-    void D3D9HLSLProgram::buildConstantDefinitions() const
+    void D3D9HLSLProgram::buildConstantDefinitions()
     {
-        mConstantDefs->floatBufferSize = mFloatLogicalToPhysical->bufferSize;
-        mConstantDefs->intBufferSize = mIntLogicalToPhysical->bufferSize;
+        mConstantDefs->bufferSize = mLogicalToPhysical->bufferSize;
 
         GpuConstantDefinitionMap::const_iterator iter = mParametersMap.begin();
         GpuConstantDefinitionMap::const_iterator iterE = mParametersMap.end();
         for (; iter != iterE ; ++iter)
         {
-            const String & paramName = iter->first;
             GpuConstantDefinition def = iter->second;
 
             mConstantDefs->map.emplace(iter->first, iter->second);
 
             // Record logical / physical mapping
-            if (def.isFloat())
-            {
-                            OGRE_LOCK_MUTEX(mFloatLogicalToPhysical->mutex);
-                mFloatLogicalToPhysical->map.emplace(def.logicalIndex,
-                        GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL));
-                mFloatLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
-            }
-            else
-            {
-                            OGRE_LOCK_MUTEX(mIntLogicalToPhysical->mutex);
-                mIntLogicalToPhysical->map.emplace(def.logicalIndex,
-                        GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL));
-                mIntLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
-            }
-
-            // Deal with array indexing
-            mConstantDefs->generateConstantDefinitionArrayEntries(paramName, def);
+            OGRE_LOCK_MUTEX(mLogicalToPhysical->mutex);
+            mLogicalToPhysical->map.emplace(def.logicalIndex,
+                    GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL, def.isFloat() ? BCT_FLOAT : BCT_INT));
+            mLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
         }
         
     }
@@ -418,22 +367,11 @@ namespace Ogre {
                 def.logicalIndex = paramIndex;
                 // populate type, array size & element size
                 populateDef(desc, def);
-                if (def.isFloat())
-                {
-                    def.physicalIndex = mFloatLogicalToPhysical->bufferSize;
-                    OGRE_LOCK_MUTEX(mFloatLogicalToPhysical->mutex);
-                    mFloatLogicalToPhysical->map.emplace(paramIndex,
-                        GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL));
-                    mFloatLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
-                }
-                else
-                {
-                    def.physicalIndex = mIntLogicalToPhysical->bufferSize;
-                    OGRE_LOCK_MUTEX(mIntLogicalToPhysical->mutex);
-                    mIntLogicalToPhysical->map.emplace(paramIndex,
-                        GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL));
-                    mIntLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
-                }
+                def.physicalIndex = mLogicalToPhysical->bufferSize*4;
+                OGRE_LOCK_MUTEX(mLogicalToPhysical->mutex);
+                mLogicalToPhysical->map.emplace(paramIndex,
+                    GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL, def.isFloat() ? BCT_FLOAT : BCT_INT));
+                mLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
 
                 if( mParametersMap.find(name) == mParametersMap.end())
                 {
@@ -583,6 +521,7 @@ namespace Ogre {
         ResourceHandle handle, const String& group, bool isManual, 
         ManualResourceLoader* loader)
         : HighLevelGpuProgram(creator, name, handle, group, isManual, loader)
+        , mEntryPoint("main")
         , mColumnMajorMatrices(true)
         , mBackwardsCompatibility(false)
         , mMicroCode(NULL)
@@ -600,9 +539,6 @@ namespace Ogre {
             dict->addParameter(ParameterDef("target", 
                 "Name of the assembler target to compile down to.",
                 PT_STRING),&msCmdTarget);
-            dict->addParameter(ParameterDef("preprocessor_defines", 
-                "Preprocessor defines use to compile the program.",
-                PT_STRING),&msCmdPreprocessorDefines);
             dict->addParameter(ParameterDef("column_major_matrices", 
                 "Whether matrix packing in column-major order.",
                 PT_BOOL),&msCmdColumnMajorMatrices);
@@ -641,7 +577,7 @@ namespace Ogre {
         if (mCompileError || !isRequiredCapabilitiesSupported())
             return false;
 
-        return GpuProgramManager::getSingleton().isSyntaxSupported(mTarget);
+        return GpuProgramManager::getSingleton().isSyntaxSupported(getTarget());
     }
     //-----------------------------------------------------------------------
     GpuProgramParametersSharedPtr D3D9HLSLProgram::createParameters(void)
@@ -657,24 +593,20 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void D3D9HLSLProgram::setTarget(const String& target)
     {
-        mTarget = "";
         std::vector<String> profiles = StringUtil::split(target, " ");
 
+        // find first supported format
         for(unsigned int i = 0 ; i < profiles.size() ; i++)
         {
             String & currentProfile = profiles[i];
             if(GpuProgramManager::getSingleton().isSyntaxSupported(currentProfile))
             {
                 mTarget = currentProfile;
-                break;
+                return;
             }
         }
 
-        if(mTarget == "")
-        {
-            LogManager::getSingleton().logMessage(
-                "Invalid target for D3D9 shader '" + mName + "' - '" + target + "'");
-        }
+        mTarget = profiles.front();
     }
 
     //-----------------------------------------------------------------------
@@ -704,15 +636,6 @@ namespace Ogre {
         static_cast<D3D9HLSLProgram*>(target)->setTarget(val);
     }
     //-----------------------------------------------------------------------
-    String D3D9HLSLProgram::CmdPreprocessorDefines::doGet(const void *target) const
-    {
-        return static_cast<const D3D9HLSLProgram*>(target)->getPreprocessorDefines();
-    }
-    void D3D9HLSLProgram::CmdPreprocessorDefines::doSet(void *target, const String& val)
-    {
-        static_cast<D3D9HLSLProgram*>(target)->setPreprocessorDefines(val);
-    }
-    //-----------------------------------------------------------------------
     String D3D9HLSLProgram::CmdColumnMajorMatrices::doGet(const void *target) const
     {
         return StringConverter::toString(static_cast<const D3D9HLSLProgram*>(target)->getColumnMajorMatrices());
