@@ -74,7 +74,6 @@ namespace Ogre
     {
         HINSTANCE hInst = mInstance;
     
-        HWND parentHWnd = 0;
         WNDPROC windowProc = DefWindowProc;
         HWND externalHandle = 0;
         mFSAAType = D3DMULTISAMPLE_NONE;
@@ -93,8 +92,9 @@ namespace Ogre
         size_t fsaaSamples = 0;
         String fsaaHint;
         bool enableDoubleClick = false;
-        int monitorIndex = -1;  //Default by detecting the adapter from left / top position
-        
+
+        D3D9RenderSystem* rsys = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+        int monitorIndex = rsys->getAdapterNumber();  // default to whatever was set in "Rendering Device" config option
 
         if(miscParams)
         {
@@ -112,15 +112,13 @@ namespace Ogre
             opt = miscParams->find("title");
             if(opt != miscParams->end())
                 title = opt->second;
-            // parentWindowHandle       -> parentHWnd
-            opt = miscParams->find("parentWindowHandle");
-            if(opt != miscParams->end())
-                parentHWnd = (HWND)StringConverter::parseSizeT(opt->second);
             opt = miscParams->find("windowProc");
             if (opt != miscParams->end())
                 windowProc = reinterpret_cast<WNDPROC>(StringConverter::parseSizeT(opt->second));
             // externalWindowHandle     -> externalHandle
             opt = miscParams->find("externalWindowHandle");
+            if (opt == miscParams->end())
+                opt = miscParams->find("parentWindowHandle");
             if(opt != miscParams->end())
                 externalHandle = (HWND)StringConverter::parseSizeT(opt->second);
             // vsync    [parseBool]
@@ -245,20 +243,13 @@ namespace Ogre
                 mWindowedWinStyle |= WS_VISIBLE;
             }
 
-            if (parentHWnd)
-            {
-                mWindowedWinStyle |= WS_CHILD;
-            }
+            if (border == "none")
+                mWindowedWinStyle |= WS_POPUP;
+            else if (border == "fixed")
+                mWindowedWinStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
+                WS_SYSMENU | WS_MINIMIZEBOX;
             else
-            {
-                if (border == "none")
-                    mWindowedWinStyle |= WS_POPUP;
-                else if (border == "fixed")
-                    mWindowedWinStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
-                    WS_SYSMENU | WS_MINIMIZEBOX;
-                else
-                    mWindowedWinStyle |= WS_OVERLAPPEDWINDOW;
-            }
+                mWindowedWinStyle |= WS_OVERLAPPEDWINDOW;
                     
             unsigned int winWidth, winHeight;
             winWidth = width;
@@ -346,7 +337,7 @@ namespace Ogre
             // Pass pointer to self
             mIsExternal = false;
             mHWnd = CreateWindowEx(dwStyleEx, "OgreD3D9Wnd", title.c_str(), getWindowStyle(fullScreen),
-                mLeft, mTop, winWidth, winHeight, parentHWnd, 0, hInst, this);
+                mLeft, mTop, winWidth, winHeight, 0, 0, hInst, this);
         }
         else
         {
@@ -729,11 +720,6 @@ namespace Ogre
             setVSyncEnabled(true);
     }
 
-    unsigned int D3D9RenderWindow::getVSyncInterval() const
-    {
-        return mVSyncInterval;
-    }
-
     void D3D9RenderWindow::reposition(int top, int left)
     {
         if (mHWnd && !mIsFullScreen)
@@ -805,6 +791,11 @@ namespace Ogre
         }
     }
 
+    PixelFormat D3D9RenderWindow::suggestPixelFormat() const
+    {
+        return mColourDepth == 16 ? PF_R5G6B5 : PF_X8R8G8B8;
+    }
+
     void D3D9RenderWindow::copyContentsToMemory(const Box& src, const PixelBox &dst, FrameBuffer buffer)
     {
         mDevice->copyContentsToMemory(this, src, dst, buffer);
