@@ -76,8 +76,6 @@ namespace Ogre {
             return GL_TEXTURE_CUBE_MAP;
         case TEX_TYPE_2D_ARRAY:
             return GL_TEXTURE_2D_ARRAY;
-        case TEX_TYPE_2D_RECT:
-            return GL_TEXTURE_RECTANGLE;
         default:
             return 0;
         };
@@ -122,39 +120,30 @@ namespace Ogre {
         mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_BASE_LEVEL, 0);
         mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_MAX_LEVEL, mNumMipmaps);
 
-        // Set some misc default parameters, these can of course be changed later.
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget,
-                                            GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget,
-                                            GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget,
-                                            GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget,
-                                            GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-
         // Set up texture swizzling.
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_R, GL_RED);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_G, GL_GREEN);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_B, GL_BLUE);
-        mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_A, GL_ALPHA);
-	
+        typedef std::array<GLint, 4> SwizzleMask;
+        SwizzleMask swizzleMask;
+
         if (PixelUtil::isLuminance(mFormat))
         {
             if (PixelUtil::getComponentCount(mFormat) == 2)
             {
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_R, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_G, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_B, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_A, GL_GREEN);
+                swizzleMask = SwizzleMask{GL_RED, GL_RED, GL_RED, GL_GREEN};
             }
             else
             {
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_R, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_G, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_B, GL_RED);
-                mRenderSystem->_getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_A, GL_ONE);
+                swizzleMask = SwizzleMask{GL_RED, GL_RED, GL_RED, GL_ONE};
             }
         }
+        else if(mFormat == PF_A8)
+        {
+            swizzleMask = SwizzleMask{GL_ZERO, GL_ZERO, GL_ZERO, GL_RED};
+        }
+        else
+        {
+            swizzleMask = SwizzleMask{GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA};
+        }
+        OGRE_CHECK_GL_ERROR(glTexParameteriv(texTarget, GL_TEXTURE_SWIZZLE_RGBA, swizzleMask.data()));
 
         GLenum format = GL3PlusPixelUtil::getGLInternalFormat(mFormat, mHwGamma);
         GLenum datatype = GL3PlusPixelUtil::getGLOriginDataType(mFormat);
@@ -200,15 +189,6 @@ namespace Ogre {
                                                                size,
                                                                NULL));
                     break;
-                case TEX_TYPE_2D_RECT:
-                    OGRE_CHECK_GL_ERROR(glCompressedTexImage2D(GL_TEXTURE_RECTANGLE,
-                                                               mip,
-                                                               format,
-                                                               width, height,
-                                                               0,
-                                                               size,
-                                                               NULL));
-                    break;
                 case TEX_TYPE_2D_ARRAY:
                 case TEX_TYPE_3D:
                     OGRE_CHECK_GL_ERROR(glCompressedTexImage3D(texTarget, mip, format,
@@ -250,17 +230,12 @@ namespace Ogre {
                     OGRE_CHECK_GL_ERROR(glTexStorage1D(GL_TEXTURE_1D, GLsizei(mNumMipmaps+1), format, GLsizei(width)));
                     break;
                 case TEX_TYPE_2D:
-                case TEX_TYPE_2D_RECT:
-                    OGRE_CHECK_GL_ERROR(glTexStorage2D(GL_TEXTURE_2D, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height)));
-                    break;
                 case TEX_TYPE_CUBE_MAP:
-                    OGRE_CHECK_GL_ERROR(glTexStorage2D(GL_TEXTURE_CUBE_MAP, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height)));
+                    OGRE_CHECK_GL_ERROR(glTexStorage2D(texTarget, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height)));
                     break;
                 case TEX_TYPE_2D_ARRAY:
-                    OGRE_CHECK_GL_ERROR(glTexStorage3D(GL_TEXTURE_2D_ARRAY, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height), GLsizei(depth)));
-                    break;
                 case TEX_TYPE_3D:
-                    OGRE_CHECK_GL_ERROR(glTexStorage3D(GL_TEXTURE_3D, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height), GLsizei(depth)));
+                    OGRE_CHECK_GL_ERROR(glTexStorage3D(texTarget, GLsizei(mNumMipmaps+1), format, GLsizei(width), GLsizei(height), GLsizei(depth)));
                     break;
                 case TEX_TYPE_EXTERNAL_OES:
                     OGRE_EXCEPT(
@@ -299,16 +274,7 @@ namespace Ogre {
                                                          originFormat, datatype, NULL));
                         break;
                     case TEX_TYPE_2D:
-                        OGRE_CHECK_GL_ERROR(glTexImage2D(GL_TEXTURE_2D,
-                                                         mip,
-                                                         format,
-                                                         width, height,
-                                                         0,
-                                                         originFormat,
-                                                         datatype, NULL));
-                        break;
-                    case TEX_TYPE_2D_RECT:
-                        OGRE_CHECK_GL_ERROR(glTexImage2D(GL_TEXTURE_RECTANGLE,
+                        OGRE_CHECK_GL_ERROR(glTexImage2D(texTarget,
                                                          mip,
                                                          format,
                                                          width, height,
@@ -385,11 +351,11 @@ namespace Ogre {
     {
         mSurfaceList.clear();
 
-        size_t depth = mDepth;
+        uint32 depth = mDepth;
         for (uint8 face = 0; face < getNumFaces(); face++)
         {
-            size_t width = mWidth;
-            size_t height = mHeight;
+            uint32 width = mWidth;
+            uint32 height = mHeight;
 
             for (uint32 mip = 0; mip <= getNumMipmaps(); mip++)
             {
