@@ -279,12 +279,9 @@ public:
     void createSceneManager()
     {
         mSceneMgr = mRoot->createSceneManager("BspSceneManager");
-        // Currently PlayPen does not use RTSS, but for now RTSS is
-        // the only system that correctly creates shaders for BSP levels
-        // loaded from PK3 archives.  Remedy this before enabling BSP test.
-        // #ifdef INCLUDE_RTSHADER_SYSTEM
-        //     mShaderGenerator->addSceneManager(mSceneMgr);
-        // #endif
+#ifdef OGRE_BUILD_COMPONENT_RTSHADERSYSTEM
+        mShaderGenerator->addSceneManager(mSceneMgr);
+#endif
     }
 
 protected:
@@ -421,7 +418,7 @@ class _OgreSampleClassExport PlayPen_ClearScene : public VisualTest
 public:
 
     PlayPen_ClearScene();
-    bool isScreenshotFrame(unsigned int frame);
+    bool isScreenshotFrame(int frame) override;
 
 protected:
     void setupContent();
@@ -580,11 +577,15 @@ public:
     void createSceneManager()
     {
         mSceneMgr = mRoot->createSceneManager("BspSceneManager");
+#ifdef OGRE_BUILD_COMPONENT_RTSHADERSYSTEM
+        mShaderGenerator->addSceneManager(mSceneMgr);
+#endif
     }
 
 protected:
 
     void setupContent();
+    void cleanupContent();
 
 };
 
@@ -772,6 +773,18 @@ protected:
 
 };
 
+class _OgreSampleClassExport PlayPen_AlphaTex : public VisualTest
+{
+public:
+
+    PlayPen_AlphaTex();
+
+protected:
+
+    void setupContent();
+
+};
+
 //------------------------------------------------------------------------------
 /** Tests DXT1 loading */
 class _OgreSampleClassExport PlayPen_Dxt1 : public VisualTest
@@ -912,20 +925,6 @@ protected:
 
 };
 
-//------------------------------------------------------------------------------
-/** Tests suppressed shadows */
-class _OgreSampleClassExport PlayPen_SuppressedShadows : public VisualTest
-{
-public:
-
-    PlayPen_SuppressedShadows();
-
-protected:
-
-    void setupContent();
-
-};
-
 //------------------------------------------------------------------------------
 /** Tests excluding shadows from a viewport */
 class _OgreSampleClassExport PlayPen_ViewportNoShadows : public VisualTest
@@ -1076,9 +1075,10 @@ public:
 
     PlayPen_VertexTexture();
 
-    virtual Ogre::String getRequiredRenderSystem()
+    virtual void testCapabilities(const RenderSystemCapabilities* caps)
     {
-        return "Direct3D9 Rendering Subsystem";
+        if(!caps->isShaderProfileSupported("hlsl") && !caps->isShaderProfileSupported("glsl"))
+            OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "not implemented");
     }
 
 protected:
@@ -1123,12 +1123,6 @@ public:
 
     PlayPen_TextureShadowsTransparentCaster();
 
-    void testCapabilities(const Ogre::RenderSystemCapabilities* caps)
-    {
-        if(!caps->hasCapability(RSC_FIXED_FUNCTION))
-            throw Ogre::Exception(999, "API doesn't support fixed function.", "testCapabilities");
-    }
-
 protected:
 
     void setupContent();
@@ -1438,20 +1432,6 @@ protected:
 
 };
 
-//------------------------------------------------------------------------------
-/** Tests simple image operations */
-class _OgreSampleClassExport PlayPen_ImageCombine : public VisualTest
-{
-public:
-
-    PlayPen_ImageCombine();
-
-protected:
-
-    void setupContent();
-
-};
-
 //------------------------------------------------------------------------------
 /** Tests non-unique resource names in different resource groups */
 class _OgreSampleClassExport PlayPen_NonUniqueResourceNames : public VisualTest
@@ -1595,6 +1575,14 @@ protected:
 
 };
 
+class _OgreSampleClassExport PlayPen_BillboardText : public VisualTest
+{
+public:
+    PlayPen_BillboardText();
+
+protected:
+    void setupContent();
+};
 //------------------------------------------------------------------------------
 
 // some common tasks for these tests:
